@@ -49,29 +49,32 @@ Implementation
 """
 
 from collections import namedtuple
-from contextlib import contextmanager
+from contextlib import ContextDecorator, contextmanager
 import re
-from typing import Dict, List
+from typing import Any, Callable, Dict, Iterable, List, Optional, Tuple, Type, cast
 import warnings
 
+from pandas._typing import F
+
 DeprecatedOption = namedtuple("DeprecatedOption", "key msg rkey removal_ver")
 RegisteredOption = namedtuple("RegisteredOption", "key defval doc validator cb")
 
-# holds deprecated option metdata
-_deprecated_options = {}  # type: Dict[str, DeprecatedOption]
+# holds deprecated option metadata
+_deprecated_options: Dict[str, DeprecatedOption] = {}
 
-# holds registered option metdata
-_registered_options = {}  # type: Dict[str, RegisteredOption]
+# holds registered option metadata
+_registered_options: Dict[str, RegisteredOption] = {}
 
 # holds the current values for registered options
-_global_config = {}  # type: Dict[str, str]
+_global_config: Dict[str, Any] = {}
 
 # keys which have a special meaning
-_reserved_keys = ["all"]  # type: List[str]
+_reserved_keys: List[str] = ["all"]
 
 
 class OptionError(AttributeError, KeyError):
-    """Exception for pandas.options, backwards compatible with KeyError
+    """
+    Exception for pandas.options, backwards compatible with KeyError
     checks
     """
 
@@ -80,12 +83,12 @@ class OptionError(AttributeError, KeyError):
 # User API
 
 
-def _get_single_key(pat, silent):
+def _get_single_key(pat: str, silent: bool) -> str:
     keys = _select_options(pat)
     if len(keys) == 0:
         if not silent:
             _warn_if_deprecated(pat)
-        raise OptionError("No such keys(s): {pat!r}".format(pat=pat))
+        raise OptionError(f"No such keys(s): {repr(pat)}")
     if len(keys) > 1:
         raise OptionError("Pattern matched multiple keys")
     key = keys[0]
@@ -98,7 +101,7 @@ def _get_single_key(pat, silent):
     return key
 
 
-def _get_option(pat, silent=False):
+def _get_option(pat: str, silent: bool = False):
     key = _get_single_key(pat, silent)
 
     # walk the nested dict
@@ -106,18 +109,18 @@ def _get_option(pat, silent=False):
     return root[k]
 
 
-def _set_option(*args, **kwargs):
+def _set_option(*args, **kwargs) -> None:
     # must at least 1 arg deal with constraints later
     nargs = len(args)
     if not nargs or nargs % 2 != 0:
-        raise ValueError("Must provide an even number of non-keyword " "arguments")
+        raise ValueError("Must provide an even number of non-keyword arguments")
 
     # default to false
     silent = kwargs.pop("silent", False)
 
     if kwargs:
-        msg = '_set_option() got an unexpected keyword argument "{kwarg}"'
-        raise TypeError(msg.format(list(kwargs.keys())[0]))
+        kwarg = list(kwargs.keys())[0]
+        raise TypeError(f'_set_option() got an unexpected keyword argument "{kwarg}"')
 
     for k, v in zip(args[::2], args[1::2]):
         key = _get_single_key(k, silent)
@@ -138,15 +141,13 @@ def _set_option(*args, **kwargs):
                 o.cb(key)
 
 
-def _describe_option(pat="", _print_desc=True):
+def _describe_option(pat: str = "", _print_desc: bool = True):
 
     keys = _select_options(pat)
     if len(keys) == 0:
         raise OptionError("No such keys(s)")
 
-    s = ""
-    for k in keys:  # filter by pat
-        s += _build_option_description(k)
+    s = "\n".join([_build_option_description(k) for k in keys])
 
     if _print_desc:
         print(s)
@@ -154,7 +155,7 @@ def _describe_option(pat="", _print_desc=True):
         return s
 
 
-def _reset_option(pat, silent=False):
+def _reset_option(pat: str, silent: bool = False) -> None:
 
     keys = _select_options(pat)
 
@@ -165,15 +166,14 @@ def _reset_option(pat, silent=False):
         raise ValueError(
             "You must specify at least 4 characters when "
             "resetting multiple keys, use the special keyword "
-            '"all" to reset all the options to their default '
-            "value"
+            '"all" to reset all the options to their default value'
         )
 
     for k in keys:
         _set_option(k, _registered_options[k].defval, silent=silent)
 
 
-def get_default_val(pat):
+def get_default_val(pat: str):
     key = _get_single_key(pat, silent=True)
     return _get_registered_option(key).defval
 
@@ -181,11 +181,11 @@ def get_default_val(pat):
 class DictWrapper:
     """ provide attribute-style access to a nested dict"""
 
-    def __init__(self, d, prefix=""):
+    def __init__(self, d: Dict[str, Any], prefix: str = ""):
         object.__setattr__(self, "d", d)
         object.__setattr__(self, "prefix", prefix)
 
-    def __setattr__(self, key, val):
+    def __setattr__(self, key: str, val: Any) -> None:
         prefix = object.__getattribute__(self, "prefix")
         if prefix:
             prefix += "."
@@ -197,21 +197,21 @@ class DictWrapper:
         else:
             raise OptionError("You can only set the value of existing options")
 
-    def __getattr__(self, key):
+    def __getattr__(self, key: str):
         prefix = object.__getattribute__(self, "prefix")
         if prefix:
             prefix += "."
         prefix += key
         try:
             v = object.__getattribute__(self, "d")[key]
-        except KeyError:
-            raise OptionError("No such option")
+        except KeyError as err:
+            raise OptionError("No such option") from err
         if isinstance(v, dict):
             return DictWrapper(v, prefix)
         else:
             return _get_option(prefix)
 
-    def __dir__(self):
+    def __dir__(self) -> Iterable[str]:
         return list(self.d.keys())
 
 
@@ -379,7 +379,7 @@ options = DictWrapper(_global_config)
 # Functions for use by pandas developers, in addition to User - api
 
 
-class option_context:
+class option_context(ContextDecorator):
     """
     Context manager to temporarily set options in the `with` statement context.
 
@@ -387,15 +387,14 @@ class option_context:
 
     Examples
     --------
-
     >>> with option_context('display.max_rows', 10, 'display.max_columns', 5):
     ...     ...
     """
 
     def __init__(self, *args):
-        if not (len(args) % 2 == 0 and len(args) >= 2):
+        if len(args) % 2 != 0 or len(args) < 2:
             raise ValueError(
-                "Need to invoke as" " option_context(pat, val, [(pat, val), ...])."
+                "Need to invoke as option_context(pat, val, [(pat, val), ...])."
             )
 
         self.ops = list(zip(args[::2], args[1::2]))
@@ -412,40 +411,46 @@ class option_context:
                 _set_option(pat, val, silent=True)
 
 
-def register_option(key, defval, doc="", validator=None, cb=None):
-    """Register an option in the package-wide pandas config object
+def register_option(
+    key: str,
+    defval: object,
+    doc: str = "",
+    validator: Optional[Callable[[Any], Any]] = None,
+    cb: Optional[Callable[[str], Any]] = None,
+) -> None:
+    """
+    Register an option in the package-wide pandas config object
 
     Parameters
     ----------
-    key       - a fully-qualified key, e.g. "x.y.option - z".
-    defval    - the default value of the option
-    doc       - a string description of the option
-    validator - a function of a single argument, should raise `ValueError` if
-                called with a value which is not a legal value for the option.
-    cb        - a function of a single argument "key", which is called
-                immediately after an option value is set/reset. key is
-                the full name of the option.
-
-    Returns
-    -------
-    Nothing.
+    key : str
+        Fully-qualified key, e.g. "x.y.option - z".
+    defval : object
+        Default value of the option.
+    doc : str
+        Description of the option.
+    validator : Callable, optional
+        Function of a single argument, should raise `ValueError` if
+        called with a value which is not a legal value for the option.
+    cb
+        a function of a single argument "key", which is called
+        immediately after an option value is set/reset. key is
+        the full name of the option.
 
     Raises
     ------
     ValueError if `validator` is specified and `defval` is not a valid value.
 
     """
-    import tokenize
     import keyword
+    import tokenize
 
     key = key.lower()
 
     if key in _registered_options:
-        msg = "Option '{key}' has already been registered"
-        raise OptionError(msg.format(key=key))
+        raise OptionError(f"Option '{key}' has already been registered")
     if key in _reserved_keys:
-        msg = "Option '{key}' is a reserved key"
-        raise OptionError(msg.format(key=key))
+        raise OptionError(f"Option '{key}' is a reserved key")
 
     # the default value should be legal
     if validator:
@@ -455,13 +460,14 @@ def register_option(key, defval, doc="", validator=None, cb=None):
     path = key.split(".")
 
     for k in path:
-        if not bool(re.match("^" + tokenize.Name + "$", k)):
-            raise ValueError("{k} is not a valid identifier".format(k=k))
+        if not re.match("^" + tokenize.Name + "$", k):
+            raise ValueError(f"{k} is not a valid identifier")
         if keyword.iskeyword(k):
-            raise ValueError("{k} is a python keyword".format(k=k))
+            raise ValueError(f"{k} is a python keyword")
 
     cursor = _global_config
     msg = "Path prefix to option '{option}' is already an option"
+
     for i, p in enumerate(path[:-1]):
         if not isinstance(cursor, dict):
             raise OptionError(msg.format(option=".".join(path[:i])))
@@ -480,7 +486,9 @@ def register_option(key, defval, doc="", validator=None, cb=None):
     )
 
 
-def deprecate_option(key, msg=None, rkey=None, removal_ver=None):
+def deprecate_option(
+    key: str, msg: Optional[str] = None, rkey: Optional[str] = None, removal_ver=None
+) -> None:
     """
     Mark option `key` as deprecated, if code attempts to access this option,
     a warning will be produced, using `msg` if given, or a default message
@@ -493,37 +501,31 @@ def deprecate_option(key, msg=None, rkey=None, removal_ver=None):
 
     Parameters
     ----------
-    key - the name of the option to be deprecated. must be a fully-qualified
-          option name (e.g "x.y.z.rkey").
-
-    msg - (Optional) a warning message to output when the key is referenced.
-          if no message is given a default message will be emitted.
-
-    rkey - (Optional) the name of an option to reroute access to.
-           If specified, any referenced `key` will be re-routed to `rkey`
-           including set/get/reset.
-           rkey must be a fully-qualified option name (e.g "x.y.z.rkey").
-           used by the default message if no `msg` is specified.
-
-    removal_ver - (Optional) specifies the version in which this option will
-                  be removed. used by the default message if no `msg`
-                  is specified.
-
-    Returns
-    -------
-    Nothing
+    key : str
+        Name of the option to be deprecated.
+        must be a fully-qualified option name (e.g "x.y.z.rkey").
+    msg : str, optional
+        Warning message to output when the key is referenced.
+        if no message is given a default message will be emitted.
+    rkey : str, optional
+        Name of an option to reroute access to.
+        If specified, any referenced `key` will be
+        re-routed to `rkey` including set/get/reset.
+        rkey must be a fully-qualified option name (e.g "x.y.z.rkey").
+        used by the default message if no `msg` is specified.
+    removal_ver : optional
+        Specifies the version in which this option will
+        be removed. used by the default message if no `msg` is specified.
 
     Raises
     ------
-    OptionError - if key has already been deprecated.
-
+    OptionError
+        If the specified key has already been deprecated.
     """
-
     key = key.lower()
 
     if key in _deprecated_options:
-        msg = "Option '{key}' has already been defined as deprecated."
-        raise OptionError(msg.format(key=key))
+        raise OptionError(f"Option '{key}' has already been defined as deprecated.")
 
     _deprecated_options[key] = DeprecatedOption(key, msg, rkey, removal_ver)
 
@@ -532,12 +534,12 @@ def deprecate_option(key, msg=None, rkey=None, removal_ver=None):
 # functions internal to the module
 
 
-def _select_options(pat):
-    """returns a list of keys matching `pat`
+def _select_options(pat: str) -> List[str]:
+    """
+    returns a list of keys matching `pat`
 
     if pat=="all", returns all registered options
     """
-
     # short-circuit for exact key
     if pat in _registered_options:
         return [pat]
@@ -550,7 +552,7 @@ def _select_options(pat):
     return [k for k in keys if re.search(pat, k, re.I)]
 
 
-def _get_root(key):
+def _get_root(key: str) -> Tuple[Dict[str, Any], str]:
     path = key.split(".")
     cursor = _global_config
     for p in path[:-1]:
@@ -558,14 +560,13 @@ def _get_root(key):
     return cursor, path[-1]
 
 
-def _is_deprecated(key):
+def _is_deprecated(key: str) -> bool:
     """ Returns True if the given option has been deprecated """
-
     key = key.lower()
     return key in _deprecated_options
 
 
-def _get_deprecated_option(key):
+def _get_deprecated_option(key: str):
     """
     Retrieves the metadata for a deprecated option, if `key` is deprecated.
 
@@ -573,7 +574,6 @@ def _get_deprecated_option(key):
     -------
     DeprecatedOption (namedtuple) if key is deprecated, None otherwise
     """
-
     try:
         d = _deprecated_options[key]
     except KeyError:
@@ -582,7 +582,7 @@ def _get_deprecated_option(key):
         return d
 
 
-def _get_registered_option(key):
+def _get_registered_option(key: str):
     """
     Retrieves the option metadata if `key` is a registered option.
 
@@ -593,12 +593,11 @@ def _get_registered_option(key):
     return _registered_options.get(key)
 
 
-def _translate_key(key):
+def _translate_key(key: str) -> str:
     """
     if key id deprecated and a replacement key defined, will return the
     replacement key, otherwise returns `key` as - is
     """
-
     d = _get_deprecated_option(key)
     if d:
         return d.rkey or key
@@ -606,7 +605,7 @@ def _translate_key(key):
         return key
 
 
-def _warn_if_deprecated(key):
+def _warn_if_deprecated(key: str) -> bool:
     """
     Checks if `key` is a deprecated option and if so, prints a warning.
 
@@ -614,18 +613,17 @@ def _warn_if_deprecated(key):
     -------
     bool - True if `key` is deprecated, False otherwise.
     """
-
     d = _get_deprecated_option(key)
     if d:
         if d.msg:
             print(d.msg)
             warnings.warn(d.msg, FutureWarning)
         else:
-            msg = "'{key}' is deprecated".format(key=key)
+            msg = f"'{key}' is deprecated"
             if d.removal_ver:
-                msg += " and will be removed in {version}".format(version=d.removal_ver)
+                msg += f" and will be removed in {d.removal_ver}"
             if d.rkey:
-                msg += ", please use '{rkey}' instead.".format(rkey=d.rkey)
+                msg += f", please use '{d.rkey}' instead."
             else:
                 msg += ", please refrain from using it."
 
@@ -634,13 +632,12 @@ def _warn_if_deprecated(key):
     return False
 
 
-def _build_option_description(k):
+def _build_option_description(k: str) -> str:
     """ Builds a formatted description of a registered option and prints it """
-
     o = _get_registered_option(k)
     d = _get_deprecated_option(k)
 
-    s = "{k} ".format(k=k)
+    s = f"{k} "
 
     if o.doc:
         s += "\n".join(o.doc.strip().split("\n"))
@@ -648,25 +645,23 @@ def _build_option_description(k):
         s += "No description available."
 
     if o:
-        s += "\n    [default: {default}] [currently: {current}]".format(
-            default=o.defval, current=_get_option(k, True)
-        )
+        s += f"\n    [default: {o.defval}] [currently: {_get_option(k, True)}]"
 
     if d:
+        rkey = d.rkey or ""
         s += "\n    (Deprecated"
-        s += ", use `{rkey}` instead.".format(rkey=d.rkey if d.rkey else "")
+        s += f", use `{rkey}` instead."
         s += ")"
 
     return s
 
 
-def pp_options_list(keys, width=80, _print=False):
+def pp_options_list(keys: Iterable[str], width=80, _print: bool = False):
     """ Builds a concise listing of available options, grouped by prefix """
-
-    from textwrap import wrap
     from itertools import groupby
+    from textwrap import wrap
 
-    def pp(name, ks):
+    def pp(name: str, ks: Iterable[str]) -> List[str]:
         pfx = "- " + name + ".[" if name else ""
         ls = wrap(
             ", ".join(ks),
@@ -679,7 +674,7 @@ def pp_options_list(keys, width=80, _print=False):
             ls[-1] = ls[-1] + "]"
         return ls
 
-    ls = []
+    ls: List[str] = []
     singles = [x for x in sorted(keys) if x.find(".") < 0]
     if singles:
         ls += pp("", singles)
@@ -701,15 +696,16 @@ def pp_options_list(keys, width=80, _print=False):
 
 @contextmanager
 def config_prefix(prefix):
-    """contextmanager for multiple invocations of API with a common prefix
+    """
+    contextmanager for multiple invocations of API with a common prefix
 
     supported API functions: (register / get / set )__option
 
     Warning: This is not thread - safe, and won't work properly if you import
     the API functions into your module using the "from x import y" construct.
 
-    Example:
-
+    Example
+    -------
     import pandas._config.config as cf
     with cf.config_prefix("display.font"):
         cf.register_option("color", "red")
@@ -723,18 +719,17 @@ def config_prefix(prefix):
     will register options "display.font.color", "display.font.size", set the
     value of "display.font.size"... and so on.
     """
-
     # Note: reset_option relies on set_option, and on key directly
     # it does not fit in to this monkey-patching scheme
 
     global register_option, get_option, set_option, reset_option
 
-    def wrap(func):
-        def inner(key, *args, **kwds):
-            pkey = "{prefix}.{key}".format(prefix=prefix, key=key)
+    def wrap(func: F) -> F:
+        def inner(key: str, *args, **kwds):
+            pkey = f"{prefix}.{key}"
             return func(pkey, *args, **kwds)
 
-        return inner
+        return cast(F, inner)
 
     _register_option = register_option
     _get_option = get_option
@@ -752,7 +747,7 @@ def config_prefix(prefix):
 # arg in register_option
 
 
-def is_type_factory(_type):
+def is_type_factory(_type: Type[Any]) -> Callable[[Any], None]:
     """
 
     Parameters
@@ -766,15 +761,14 @@ def is_type_factory(_type):
 
     """
 
-    def inner(x):
+    def inner(x) -> None:
         if type(x) != _type:
-            msg = "Value must have type '{typ!s}'"
-            raise ValueError(msg.format(typ=_type))
+            raise ValueError(f"Value must have type '{_type}'")
 
     return inner
 
 
-def is_instance_factory(_type):
+def is_instance_factory(_type) -> Callable[[Any], None]:
     """
 
     Parameters
@@ -791,35 +785,59 @@ def is_instance_factory(_type):
         _type = tuple(_type)
         type_repr = "|".join(map(str, _type))
     else:
-        type_repr = "'{typ}'".format(typ=_type)
+        type_repr = f"'{_type}'"
 
-    def inner(x):
+    def inner(x) -> None:
         if not isinstance(x, _type):
-            msg = "Value must be an instance of {type_repr}"
-            raise ValueError(msg.format(type_repr=type_repr))
+            raise ValueError(f"Value must be an instance of {type_repr}")
 
     return inner
 
 
-def is_one_of_factory(legal_values):
+def is_one_of_factory(legal_values) -> Callable[[Any], None]:
 
     callables = [c for c in legal_values if callable(c)]
     legal_values = [c for c in legal_values if not callable(c)]
 
-    def inner(x):
+    def inner(x) -> None:
         if x not in legal_values:
 
             if not any(c(x) for c in callables):
                 uvals = [str(lval) for lval in legal_values]
                 pp_values = "|".join(uvals)
-                msg = "Value must be one of {pp_values}"
+                msg = f"Value must be one of {pp_values}"
                 if len(callables):
                     msg += " or a callable"
-                raise ValueError(msg.format(pp_values=pp_values))
+                raise ValueError(msg)
 
     return inner
 
 
+def is_nonnegative_int(value: Optional[int]) -> None:
+    """
+    Verify that value is None or a positive int.
+
+    Parameters
+    ----------
+    value : None or int
+            The `value` to be checked.
+
+    Raises
+    ------
+    ValueError
+        When the value is not None or is a negative integer
+    """
+    if value is None:
+        return
+
+    elif isinstance(value, int):
+        if value >= 0:
+            return
+
+    msg = "Value must be a nonnegative integer or None"
+    raise ValueError(msg)
+
+
 # common type validators, for convenience
 # usage: register_option(... , validator = is_int)
 is_int = is_type_factory(int)
@@ -829,7 +847,7 @@ is_str = is_type_factory(str)
 is_text = is_instance_factory((str, bytes))
 
 
-def is_callable(obj):
+def is_callable(obj) -> bool:
     """
 
     Parameters
